#ifndef _50_H_
#define _50_H_
#include<system.h>

/************************************************************
 * This is the header file that goes with the starting
 * library routines that  Drive LCD display
 *        
 */


/*  LCD Routines
 *
 *  Notes: Board is setup for 4 bit data on the upper
 *  bits of port D. Control port is on port C.2 and
 *  C.5. Note that the R/W is pulled to 0, so that only
 *  writing is done to the display.
 *  NOTE: Ports must be set digital before using the LCD display
 *  */

// Ports naming

#define LCD_DATA	latd   // LCD data port

#define LCD_E_port	latc	//enable control port NEW
#define LCD_RS_port	latc	//register select port NEW

// Control signals

#define LCD_E_bit	5	//LCD enable register
#define LCD_RS_bit	2	//LCD register select

// Some useful macros. These set or clear the control
// or register select bits.

#define LCD_E_U     set_bit(LCD_E_port, LCD_E_bit)
#define LCD_E_D     clear_bit(LCD_E_port, LCD_E_bit)
#define LCD_RS_U    set_bit(LCD_RS_port, LCD_RS_bit)
#define LCD_RS_D    clear_bit(LCD_RS_port, LCD_RS_bit)

// Use nested macros

#define write_data  LCD_RS_U; LCD_E_U; LCD_E_D;  // RS=1, toggle enable bit
#define write_cmd   LCD_RS_D; LCD_E_U; LCD_E_D;  // RS=0, toggle enable bit

// LCD Function prototypes 
void LCD_init(void);     // init LCD display
void LCD_ph1(char data);  // sends ascii of a single hex digit to display
void LCD_char(char data); // sends ascii character to display
void LCD_icmd(char data); //  sends 8 bit command (init till set to 4 bits)
void LCD_cmd(char data); // sends command to 4 bit interface nibble by nibble
void LCD_dec(char data); // displays byte as decimal number
void LCD_dec(short data);
void LCD_dec(unsigned short data);
void LCD_hex(char data);  // displays byte as hex number
void LCD_hex(short data);  // displays byte as hex number
void LCD_hex(unsigned short data);  // displays byte as hex number
void LCD_int(char data);
void LCD_int(short data);
void LCD_int(unsigned short data);
void LCD_nib(char data);
void LCD_bin(char data);  // displays byte as binary number
void LCD_bin(unsigned short data);  // displays byte as binary number
void LCD_init(void);     //initializes LDC display
void LCD_printf( const char* text ); //sends ascii string to display
void LCD_setpos(char line, char pos); //sets potition on display
void LCD_clear(void);    // clear LCD Screen


//Terminal Function prototypes
void term_init(void);  					//Initializes mc for serial i/o
void putc(char value); 					//send ascii character to terminal
char getc(void);	   					//get ascii character from terminal
void term_char(char data);				//prints a character to the terminal
void term_printf( const char* text );	//prints a character string to the terminal via the serial port
void term_dec(unsigned short dat);		//prints value of unsigned short variable to terminal
unsigned short getnum(void);			//prints value of unsigned short value
void term_menu(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);				//dumps memory contents into terminal
void dump_mem(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);

//Interrupts
void int_wdt_init(void);			//automatically sets the proper interrupt flags
void int_wdt_init_sleep(void);			

//Serial Protocols
void spi_init(void);																//establishes settings for SPI interface.
void i2c_init(void);																//establishes settings for I2C interface.

void i2c_start(void);																//These functions, internal to the 
void i2c_stop(void)	;																//main I2C functions, provide short						
unsigned short i2c_send_ack(void);															//hand for common actions.
void i2c_send_not_ack(void);
void i2c_receive_mode(void);
										
unsigned short i2c_time(bool write, unsigned short address_low, unsigned short address_high);	//displays time to LCD, with option of memory write
unsigned short i2c_data(bool car_on, unsigned short over, unsigned short fault, unsigned short address_low_inc, unsigned short address_high_inc);				//writes all source measurements to memory															
unsigned short mem_write_byte(unsigned short address_low, unsigned short address_high, short value); //writes byte to memory
void time_init_clk(unsigned short minutes, unsigned short hours, unsigned short date, unsigned short month );			//initializes the clock to proper time
void time_init_set(void);															//Sets alarm and other setting registers on time chip
unsigned short address_update_low(unsigned short address_low);									//updates where the MC will write to memory next

//AD Conversion
void ad_init(void);							//sets register bits to allow AD conversion
unsigned short check_ad(short pin_select);			//gives battery voltage using AD pin reading
unsigned short volt_scaler(unsigned short raw);		// Scales unsigned short raw data to range of 0-5
void print_ad_LCD(unsigned short dingus);				// Prints the scaled voltage to the LCD
void print_ad_term_volt(unsigned short dingus);		// Prints a voltage to the HyperTerminal screen
void print_ad_term_amp(unsigned short dingus, unsigned short type);	// Prints a current to the HyperTerminal screen
unsigned short soc_check(void);					//Check's the battery's state of charge.
		
				
//Switching
void io_init(void);							//sets pins to either digital inputs or outputs
unsigned short control_routine(unsigned short over, bool car_on, unsigned short wait_time);		//determines current electrical systems status
									//pulls appropriate switches; reports on/off
													//status of various switches to main.
//Killing Time
void sleep_mode(void);
short wait_mode(short mode);


// LCD Menu functions
void LCD_main_menu(bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);
void menu_init(void);
void disp_menu_option(short option);
short run_option(short option, bool read_all, unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);

// Load Check - LCD - functions
void load_check(void);

// Set Clock - LCD - functions
void set_time(void);
unsigned short bin_to_bcd(unsigned short bin);
short check_twentyfour(void);
void write_twentyfour(short tf);
unsigned short check_termread(void);
void write_term_read(unsigned short tr);

// Set Override Mode - LCD - functions
void set_override(void);
unsigned short check_override(void);
void write_override(unsigned short ovr);

// timer0 - time out from menu - related functions
void time_out_init(void);
short check_timeoutflag(void);
short write_timeoutflag(short tof);
unsigned short timer_count(unsigned short count, unsigned short time_lim);
short timer_bust(unsigned short count, unsigned short time_lim);
unsigned short reset_timer(void);


#endif //_EESD_H_
